@@ -1,6 +1,6 @@
 #' Build a Big Data Random Forest Model
 #'
-#' Builds a Random Forest Model on an H2O Frame
+#' Builds a Random Forest Model on an H2OFrame
 #'
 #' @param x A vector containing the names or indices of the predictor variables
 #'        to use in building the GBM model.
@@ -8,16 +8,18 @@
 #'        contain a header, this is the column index number starting at 1, and
 #'        increasing from left to right. (The response must be either an integer
 #'        or a categorical variable).
-#' @param training_frame An H2O Frame object containing the
+#' @param training_frame An H2OFrame object containing the
 #'        variables in the model.
 #' @param model_id (Optional) The unique id assigned to the resulting model. If
 #'        none is given, an id will automatically be generated.
-#' @param validation_frame An H2O Frame object containing the variables in the model.  Default is NULL.
+#' @param validation_frame An H2OFrame object containing the variables in the model.  Default is NULL.
 #' @param checkpoint "Model checkpoint (either key or H2ODeepLearningModel) to resume training with."
+#' @param ignore_const_cols A logical value indicating whether or not to ignore all the constant columns in the training frame.
 #' @param mtries Number of variables randomly sampled as candidates at each split.
 #'        If set to -1, defaults to sqrt{p} for classification, and p/3 for regression,
 #'        where p is the number of predictors.
 #' @param sample_rate Sample rate, from 0 to 1.0.
+#' @param col_sample_rate_per_tree Column sample rate per tree (from \code{0.0} to \code{1.0})
 #' @param build_tree_one_node Run on one node only; no network overhead but
 #'        fewer cpus used.  Suitable for small datasets.
 #' @param ntrees A nonnegative integer that determines the number of trees to
@@ -44,6 +46,7 @@
 #'        Must be "AUTO", "Random" or "Modulo"
 #' @param keep_cross_validation_predictions Whether to keep the predictions of the cross-validation models
 #' @param score_each_iteration Attempts to score each tree.
+#' @param score_tree_interval Score the model after every so many trees. Disabled if set to 0.
 #' @param stopping_rounds Early stopping based on convergence of stopping_metric.
 #'        Stop if simple moving average of length k of the stopping_metric does not improve
 #'        (by stopping_tolerance) for k=stopping_rounds scoring events.
@@ -52,6 +55,7 @@
 #'        Can be one of "AUTO", "deviance", "logloss", "MSE", "AUC", "r2", "misclassification".
 #' @param stopping_tolerance Relative tolerance for metric-based stopping criterion (if relative
 #'        improvement is not at least this much, stop)
+#' @param max_runtime_secs Maximum allowed runtime in seconds for model training. Use 0 to disable.
 #' @param ... (Currently Unimplemented)
 #' @return Creates a \linkS4class{H2OModel} object of the right type.
 #' @seealso \code{\link{predict.H2OModel}} for prediction.
@@ -59,9 +63,11 @@
 h2o.randomForest <- function(x, y, training_frame,
                              model_id,
                              validation_frame = NULL,
+                             ignore_const_cols = TRUE,
                              checkpoint,
                              mtries = -1,
                              sample_rate = 0.632,
+                             col_sample_rate_per_tree = 1.0,
                              build_tree_one_node = FALSE,
                              ntrees = 50,
                              max_depth = 20,
@@ -80,22 +86,24 @@ h2o.randomForest <- function(x, y, training_frame,
                              fold_assignment = c("AUTO","Random","Modulo"),
                              keep_cross_validation_predictions = FALSE,
                              score_each_iteration = FALSE,
+                             score_tree_interval = 0,
                              stopping_rounds=0,
                              stopping_metric=c("AUTO", "deviance", "logloss", "MSE", "AUC", "r2", "misclassification"),
-                             stopping_tolerance=1e-3
+                             stopping_tolerance=1e-3,
+                             max_runtime_secs=0
                              )
 {
-  # Training_frame and validation_frame may be a key or a Frame object
-  if (!is.Frame(training_frame))
+  # Training_frame and validation_frame may be a key or an H2OFrame object
+  if (!is.H2OFrame(training_frame))
     tryCatch(training_frame <- h2o.getFrame(training_frame),
              error = function(err) {
-               stop("argument \"training_frame\" must be a valid Frame or key")
+               stop("argument \"training_frame\" must be a valid H2OFrame or key")
              })
   if (!is.null(validation_frame)) {
-    if (!is.Frame(validation_frame))
+    if (!is.H2OFrame(validation_frame))
         tryCatch(validation_frame <- h2o.getFrame(validation_frame),
                  error = function(err) {
-                   stop("argument \"validation_frame\" must be a valid Frame or key")
+                   stop("argument \"validation_frame\" must be a valid H2OFrame or key")
                  })
   }
 
@@ -112,12 +120,16 @@ h2o.randomForest <- function(x, y, training_frame,
     parms$model_id <- model_id
   if(!missing(validation_frame))
     parms$validation_frame <- validation_frame
+  if(!missing(ignore_const_cols))
+    parms$ignore_const_cols <- ignore_const_cols
   if(!missing(checkpoint))
     parms$checkpoint <- checkpoint
   if(!missing(mtries))
     parms$mtries <- mtries
   if(!missing(sample_rate))
     parms$sample_rate <- sample_rate
+  if(!missing(col_sample_rate_per_tree))
+    parms$col_sample_rate_per_tree <- col_sample_rate_per_tree
   if(!missing(build_tree_one_node))
     parms$build_tree_one_node <- build_tree_one_node
   if(!missing(binomial_double_trees))
@@ -147,11 +159,12 @@ h2o.randomForest <- function(x, y, training_frame,
   if( !missing(fold_column) )               parms$fold_column            <- fold_column
   if( !missing(fold_assignment) )           parms$fold_assignment        <- fold_assignment
   if( !missing(keep_cross_validation_predictions) )  parms$keep_cross_validation_predictions  <- keep_cross_validation_predictions
-  if (!missing(score_each_iteration))
-    parms$score_each_iteration <- score_each_iteration
+  if (!missing(score_each_iteration)) parms$score_each_iteration <- score_each_iteration
+  if (!missing(score_tree_interval)) parms$score_tree_interval <- score_tree_interval
   if(!missing(stopping_rounds)) parms$stopping_rounds <- stopping_rounds
   if(!missing(stopping_metric)) parms$stopping_metric <- stopping_metric
   if(!missing(stopping_tolerance)) parms$stopping_tolerance <- stopping_tolerance
+  if(!missing(max_runtime_secs)) parms$max_runtime_secs <- max_runtime_secs
 
   .h2o.modelJob('drf', parms)
 }
